/** 
 *******************************************************************************
 * @file    main.c
 * @brief   CAN Sample Application.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "txz_sample_def.h"
#include "can.h"
#include "bsp.h"
#include "bsp_uart_io.h"
#include "bsp_timer.h"
#include "bsp_adc.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup CAN CAN Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CAN_Private_macro CAN Private Macro
 *  @{
 */
/** 
 *  @defgroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_OUTPUT_COUNT_TIME             ((uint32_t)(5000))

#define MAIN_NULL       ((void *)0)     /*!< NULL. */
#define USER_ID_UART    ((uint32_t)1)   /*!< User ID :USB UART (Any value) */

/** 
 *  @}
 */ /* End of group DataLengtheConfiguration */
/** 
 *  @defgroup MSG_ID_CFG Message ID  Configuration
 *  @brief    Message ID  Configuration.
 *  @{
 */
#define MSG_BOX_TX_BITNO            (0x00000001U)
#define MSG_BOX_RX_BITNO            (0x00000400U)
/** 
 *  @}
 */ /* End of group MSG_ID_CFG */
 
/** 
 *  @}
 */ /* End of group CAN_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CAN_Private_define CAN Private Define
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,          /*!< Idle.             */
    PROC_START,         /*!< Start.            */
    PROC_ADC_START,     /*!< ADC Start.        */
    PROC_GET_ADCVALUE,  /*!< Get ADC value.    */
    PROC_CAN_TRANS,     /*!< CAN Transfor      */
    PROC_ADC_STOP,      /*!< ADC Stop.         */
    PROC_OUTPUT,        /*!< Output.           */
    PROC_ERROR          /*!< Error.            */
} Processing;

/** 
 *  @}
 */ /* End of group CAN_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CAN_Private_typedef CAN Private Typedef
 *  @{
 */
 /*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/
typedef struct
{
    t32a_t      t32a;              /*!< Driver      :T32A.     */
    adc_t       adc;               /*!< Driver      :ADC.      */
    uart_io_t   *p_uart;           /*!< Application :USB UART. */
    timer_t     timer;             /*!< Application :Timer (1ms timer). */
} instance_t;

/*----------------------------------*/
/** 
 * @brief Information.
*/
/*----------------------------------*/
typedef struct
{
   TXZ_WorkState       adc_sts;    /*!< Adc check status   */
   TXZ_FunctionalState update;     /*!< Update flag.       */
   uint32_t            count;      /*!< count flag.        */
} information_t;
/** 
 *  @}
 */ /* End of group CAN_Private_define */
/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CAN_Private_variables CAN Private Variables
 *  @{
 */
static instance_t instance;     /*!< Instance.                */
static Processing proc;         /*!< Processing mode.         */
static information_t info;      /*!< Clock information.       */
static uint32_t data0 = 0;
static uint32_t rev0 = 0;
static uint8_t rev_data0[4] = {'\0'};
/**
 *  @}
 */ /* End of group CAN_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CAN_Private_fuctions CAN Private Fuctions
 *  @{
 */
static void variable_initialize(void);
static TXZ_Result driver_initialize(void);
static TXZ_Result driver_finalize(void);
static TXZ_Result application_initialize(void);
static TXZ_Result application_finalize(void);
static void timer_interval_handler(uint32_t id);
static void done_handler(uint32_t id, TXZ_Result result);

/*--------------------------------------------------*/
/** 
  * @brief  Variable Initialize
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void variable_initialize(void)
{
    /* Instance. */
    instance.t32a.p_instance    = MAIN_NULL;
    instance.adc.p_instance     = MAIN_NULL;
    instance.p_uart             = MAIN_NULL;
    instance.timer.init.p_t32a  = MAIN_NULL;
    /* Processing mode. */
    proc = PROC_START;
    /* Information. */
    info.adc_sts = TXZ_BUSY;
    info.update  = TXZ_ENABLE;  /* Init Output. */
    info.count   = 0;
}

/*--------------------------------------------------*/
/** 
  * @brief  Driver Initialize
  * @param  -
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver initialize has done within BSP. */
    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;

        /*--- Construct  ---*/
        /* Register Allocation */
        switch (bsp_get_timer_ch(BSP_TIMER_1MS))
        {
        case 0:
            p_t32a->p_instance = TSB_T32A0;
            break;
        case 1:
            p_t32a->p_instance = TSB_T32A1;
            break;
        case 2:
            p_t32a->p_instance = TSB_T32A2;
            break;
        case 3:
            p_t32a->p_instance = TSB_T32A3;
            break;
        case 4:
            p_t32a->p_instance = TSB_T32A4;
            break;
        case 5:
            p_t32a->p_instance = TSB_T32A5;
            break;
        default:
            p_t32a->p_instance = MAIN_NULL;
            break;
        }
        /*--- Initialize ---*/
        p_t32a->init_mode.mode.halt = T32A_DBG_HALT_STOP;
        p_t32a->init_mode.mode.mode = T32A_MODE_16;
        if (p_t32a->p_instance != MAIN_NULL)
        {
            if (t32a_mode_init(p_t32a) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    /*----------------------*/
    /* Uart                 */
    /*----------------------*/
    /* Driver initialize will process within USB UART. */
    /*----------------------*/
    /* ADC                  */
    /*----------------------*/
    {
        adc_t *p_adc = &instance.adc;

        /*--- Construct  ---*/
        /* Register Allocation */
        p_adc->p_instance       = TSB_ADA;
        /*--- Initialize ---*/
        p_adc->init.id          = (uint32_t)p_adc;
        p_adc->init.clk.exaz0   = ADC_SAMPLING_PERIOD0_XN;
        p_adc->init.clk.exaz1   = ADC_SAMPLING_PERIOD1_XN;
        p_adc->init.clk.vadcld  = ADC_SCLK_5;                   /*less than 40MHz*/
        p_adc->init.clk.sampling_select |= ((ADCEXAZSEL_EXAZ1 << ADCEXAZSEL_AIN1 )|(ADCEXAZSEL_EXAZ1 << ADCEXAZSEL_AIN0 )); 
        p_adc->init.mod1        = adc_get_mode1(p_adc);
        p_adc->init.mod2        = ADC_MOD2_SET;
        p_adc->handler.single       = MAIN_NULL;
        p_adc->handler.continuity   = done_handler;
        p_adc->handler.trigger      = MAIN_NULL;
        p_adc->handler.highpriority = MAIN_NULL;
        if (adc_init(p_adc) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Driver Finalize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* ADC                  */
    /*----------------------*/
    {
        adc_t *p_adc = &instance.adc;

        /*--- Finalize ---*/
        if (adc_deinit(p_adc) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
        /*--- Destruct ---*/
        /* Register Release */
        p_adc->p_instance = MAIN_NULL;
    }
    /*----------------------*/
    /* Uart                 */
    /*----------------------*/
    /* Driver finalize has done within USB UART.    */
    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;

        /*--- Finalize ---*/
        /* no processing */
        /*--- Destruct ---*/
        /* Register Release */
        p_t32a->p_instance = MAIN_NULL;
    }
    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver finalize will process within BSP.     */

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize
  * @param  -
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Timer                */
    /*----------------------*/
    /* 1ms timer */
    {
        timer_t *p_timer = &instance.timer;

        p_timer->init.id       = (uint32_t)&instance.timer;
        p_timer->init.p_t32a   = &instance.t32a;
        p_timer->init.type     = bsp_get_timer_type(BSP_TIMER_1MS);
        p_timer->init.irq      = bsp_get_timer_nvic(BSP_TIMER_1MS);
        p_timer->init.interval = 1000;
        p_timer->init.handler  = timer_interval_handler;
        timer_initialize(p_timer);
    }
    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    /* Tx & Rx use same UART channel. */
    {
        uart_io_initial_setting_t param;

        param.id         = USER_ID_UART;
        switch (bsp_get_usb_uart_tx_ch(BSP_USB_UART_0))
        {
        case 0:
            param.p_reg  = TSB_UART0;
            break;
        case 1:
            param.p_reg  = TSB_UART1;
            break;
        case 2:
            param.p_reg  = TSB_UART2;
            break;
        default:
            param.p_reg  = MAIN_NULL;
            break;
        }
        param.boudrate   = bsp_get_usb_uart_tx_baudrate(BSP_USB_UART_0);
        param.irq.tx     = bsp_get_usb_uart_tx_nvic(BSP_USB_UART_0);
        param.irq.rx     = bsp_get_usb_uart_rx_nvic(BSP_USB_UART_0);
        param.irq.err    = bsp_get_usb_uart_err_nvic(BSP_USB_UART_0);
        if (param.p_reg != MAIN_NULL)
        {
            result  = uart_io_initialize(&param);
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    /*----------------------*/
    /* Thermistors          */
    /*----------------------*/
    {
        adc_t *p_adc = &instance.adc;
        adc_channel_setting_t param;
  
        param.interrupt = ADC_INT_ENABLE;
        param.type = ADC_CONVERSION_CNT;
        p_adc->info.ch[bsp_get_thermistors_ch(BSP_THERMISTORS_1)].init.type = ADC_CONVERSION_CNT;
        param.ain = bsp_get_thermistors_adin(BSP_THERMISTORS_1);
        if (adc_channel_setting(p_adc, bsp_get_thermistors_ch(BSP_THERMISTORS_1), &param) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Finalize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_finalize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Thermistors          */
    /*----------------------*/
    if (instance.adc.p_instance != MAIN_NULL)
    {
        if (adc_channel_clear(&instance.adc, bsp_get_thermistors_ch(BSP_THERMISTORS_1)) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }
    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    if (instance.p_uart != MAIN_NULL)
    {
        uart_io_finalize();
    }
    /*----------------------*/
    /* Timer                */
    /*----------------------*/
    if (instance.timer.init.p_t32a != MAIN_NULL)
    {
        timer_finalize(&instance.timer);
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Interval Time Handler.
  * @param  id        :User ID
  * @return -
  * @retval -
  * @note   for 1ms Timer.
  * @note   Call by Timer.
  */
/*--------------------------------------------------*/
static void timer_interval_handler(uint32_t id)
{
    if (id == (uint32_t)&instance.timer)
    {
        /* Interval Time Count. */
        info.count++;
        if (info.count >= CFG_OUTPUT_COUNT_TIME)
        {
            /*>>> Critical information. */
            info.update = TXZ_ENABLE;
            /*<<< Critical information. */
            info.count  = 0;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  ADC done hadler
  * @param  id     :User ID
  * @param  result :TXZ_SUCCESS / TXZ_ERROR
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void done_handler(uint32_t id, TXZ_Result result)
{
  info.adc_sts = TXZ_DONE;
}

/**
 *  @}
 */ /* End of group CAN_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CAN_Exported_functions CAN Exported Functions
 *  @{
 */
 
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /*--- variable    ---*/
    variable_initialize();
    /*--- driver      ---*/
    if (driver_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    /*--- application ---*/
    if (application_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }

    /*------------------------*/
    /* CAN Setting            */
    /*------------------------*/
    /* baudrate set */
    CAN_BitConfig(TSB_CANB);
    /*  Tx ch0      */
    CAN_TX_Config(TSB_CANB,TSB_CANBMB0);
    /*  Rx ch10     */
    CAN_RX_Config(TSB_CANB,TSB_CANBMB10);
  
    /*------------------------*/
    /*          Loop          */
    /*------------------------*/ 
    while(proc != PROC_ERROR){
      switch(proc)
       {
          /*--- Start ---*/
          case PROC_START:
             {
                timer_start(&instance.timer);
                proc = PROC_ADC_START;
             }
             break;

          /*--- ADC Start ---*/
          case PROC_ADC_START:
             {
                /* ADC check per 5s */
                if(info.update == TXZ_ENABLE)
                {
                    /* Start ADC Convert */
                    NVIC_EnableIRQ(INTADACNT_IRQn);
                    if (adc_start(&instance.adc) == TXZ_SUCCESS)
                    {
                      proc = PROC_GET_ADCVALUE;
                    }
                    else
                    {
                      proc = PROC_ADC_STOP;
                    }
                }
             }
             break;

          /*--- Conversion ---*/
          case PROC_GET_ADCVALUE:
             {
                if (info.adc_sts == TXZ_DONE)
                {
                    /* get adc value  */
                    if(adc_channel_get_value(&instance.adc, bsp_get_thermistors_ch(BSP_THERMISTORS_1), &data0) != TXZ_SUCCESS)
                    {
                      proc = PROC_ADC_STOP;
                    }
                    else
                    {
                      proc = PROC_CAN_TRANS;
                    }
                }
             }
             break;

          /*--- Main Stop ---*/
          case PROC_CAN_TRANS:
             {
                TSB_CANBMB0->DL = data0;	 /* Put the ADC result into mailbox0 */
                if((TSB_CANB->TRS & MSG_BOX_TX_BITNO)==0)
                {
                   TSB_CANB->TRS = MSG_BOX_TX_BITNO;   /* mailnox No.0 Start to send */
                }else{
                }
                /* Polling TX, Wait TX Ack */
                while((TSB_CANB->TA & 0x00000001) != 0x00000001)
                {
                }
                /* CANRPM Check */
                while((TSB_CANB->RMP & MSG_BOX_RX_BITNO) != MSG_BOX_RX_BITNO)
                {
                }
                /* Read Receive data */
                rev0 = TSB_CANBMB10->DL;
                /* Convert HEX to string */
                sprintf((char *)rev_data0, "%x", rev0);
                proc = PROC_OUTPUT;
             }
             break;
                
          /*--- Output ---*/
          case PROC_OUTPUT:
             {
                printf("nAIN0 > 0x%04x\r\n", rev0);
                proc = PROC_ADC_STOP;
             }
             break;

          /*--- Interval Wait ---*/
          case PROC_ADC_STOP:
             {
                /*>>> Critical information. */
                info.update = TXZ_DISABLE;
                /*<<< Critical information. */
                /* adc stop                */
                if (adc_stop(&instance.adc) != TXZ_SUCCESS)
                {
                    timer_stop(&instance.timer);
                    proc = PROC_ERROR;
                }
                else{
                    /* disable the IRQ        */
                    NVIC_DisableIRQ(INTADACNT_IRQn);
                    proc = PROC_ADC_START;
                }
             }
             break;
                    
          /*--- Error ---*/
          case PROC_ERROR:
             /* no processing */
             break;

         /*--- Default ---*/
          default:
             proc = PROC_ERROR;
             break;
        }
    }
    /*>>> System Error. Please Debug. */
    /*----------------------*/
    /* Finalize             */
    /*----------------------*/
    /*--- application ---*/
    if (application_finalize() != TXZ_SUCCESS)
    {
        /* no processing */
    }
    /*--- driver      ---*/
    if (driver_finalize() != TXZ_SUCCESS)
    {
        /* no processing */
    }
    /*----------------------*/
    /* BSP finalize         */
    /*----------------------*/
    bsp_finalize();
}
/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_rx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_tx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_err_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Timer.
  * @param  timer :Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_timer(BSPTimer timer)
{
    switch (timer)
    {
    case BSP_TIMER_1MS:
        timer_irq_handler(&instance.timer);
        break;
    default:
        /* no processing */
        break;
    }
}
/**
 *  @}
 */ /* End of group CAN_Exported_functions */

/**
 *  @}
 */ /* End of group CAN */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
