/** 
 *******************************************************************************
 * @file    main.c
 * @brief   ETHM_LoopBack Sample Application.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/**
 *  @addtogroup Sample_Appli
 *  @{
 */

/** 
 *  @addtogroup ETHM_LoopBack
 *  @{
 */

/** 
 *  @defgroup MAIN_PRIVATE  MAIN Private
 *  @brief    MAIN Private Group.
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "txz_sample_def.h"
#include "bsp.h"
#include "bsp_uart_io.h"
#include "ethm.h"
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/* no define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @}
 */ /* End of group Timer_EVENT_COUNT_Private_define */
#define MAIN_NULL          ((void *)0)     /*!< NULL. */
#define USER_ID_UART       ((uint32_t)1)   /*!< User ID :USB UART (Any value) */
#define USER_ID_ETHM_TX    ((uint32_t)2)   /*!< User ID :ETHM (Any value) */
#define USER_ID_ETHM_RX    ((uint32_t)3)   /*!< User ID :ETHM (Any value) */
/** 
 *  @defgroup ETHM_IO_SendMax  ETHM IO Send Max.
 *  @brief    Size of send max. (byte)
 *  @{
 */
#define ETHM_IO_SEND_MAX       ((uint32_t)64)
/**
 *  @}
 */ /* End of group ETHM_IO_SendMax */

/** 
 *  @defgroup ETHM_IO_ReceiveMax  ETHM IO Receive Max.
 *  @brief    Size of receive max. (byte)
 *  @{
 */
#define ETHM_IO_RECEIVE_MAX    ((uint32_t)64)
/**
 *  @}
 */ /* End of group ETHM_IO_ReceiveMax */
 
/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/*----------------------------------*/
/** 
 * @enum   Processing
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,              /*!< Idle.                        */
    PROC_OUTPUT_MSG,        /*!< Output first message.        */
    PROC_INPUT,             /*!< Wait to input.               */
    PROC_COMMAND_PARSE,       /*!< Output Echo back data.       */
    PROC_WRITE_EXEC,       /*!< Output Echo back data.       */
    PROC_READ_EXEC,       /*!< Output Echo back data.       */
    PROC_OUTPUT_ERR,        /*!< Output input error message.  */
    PROC_ERROR              /*!< Error.                       */
} Processing;

/*----------------------------------*/
/** 
 * @enum   SendState
 * @brief  Send state.
*/
/*----------------------------------*/
typedef enum {
    SEND_IDLE = 0U,     /*!< Idle                   */
    SEND_RUN,           /*!< Transmiting.           */
    SEND_SUCCESS,       /*!< Transmission complete. */
    SEND_FAILURE        /*!< Transmission error.    */
} SendState;
/*----------------------------------*/
/** 
 * @enum   ReceiveState
 * @brief  Receive state.
*/
/*----------------------------------*/
typedef enum {
    RECV_IDLE = 0U,     /*!< Idle                   */
    RECV_RUN,           /*!< Receiving              */
    RECV_SUCCESS,       /*!< Transmission complete. */
    RECV_FAILURE        /*!< Received error.        */
} ReceiveState;

/** 
 * @struct send_t
 * @brief  Send information.
*/
/*----------------------------------*/
typedef struct
{
    volatile SendState  state;                          /*!< Status.                     */
    uint32_t            wp;                             /*!< Write pointer.              */
    uint32_t            rp;                             /*!< Read pointer(Now, no use).  */
    uint8_t             buff[ETHM_IO_SEND_MAX];         /*!< Send data.                  */
} send_t;

/*----------------------------------*/
/** 
 * @struct receive_t
 * @brief  Receive information.
*/
/*----------------------------------*/
typedef struct
{
    volatile ReceiveState state;                        /*!< Status.                     */
    volatile uint32_t     wp;                           /*!< Write pointer.              */
    volatile uint32_t     rp;                           /*!< Read pointer(Now, no use).  */
    volatile uint32_t     length;                       /*!< Num of receive data.        */
    uint8_t                buff[ETHM_IO_RECEIVE_MAX];    /*!< Receive data.               */
} receive_t;

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/*----------------------------------*/
/** 
 * @struct input_t
 * @brief  Input data information.
*/
/*----------------------------------*/
typedef struct
{
    char    data[(32+1)];  /*!< Input data(+ for NULL 1byte). */
    uint8_t wp;                             /*!< Write pointer.                */
} input_t;
/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
static Processing proc;     /*!< Processing mode.        */
static input_t    input;        /*!< Input data information. */
static uint8_t read_data[UART_IO_RECEIVE_MAX];  /*!< read data buffer. */
static receive_t    receive;  /*!< Receive information.  */
send_t       send;                               /*!< Send information.     */
static gpio_t port;             /*!< Port driver instance. */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
static void port_error(void);
static void variable_initialize(void);
static void clear_data(void);
static void ETHM_port_initialize(void);
static void ethm_io_initialize(void);
static TXZ_Result application_initialize(void);

/*--------------------------------------------------*/
/** 
  * @brief  Error Hook
  * @param  -
  * @return -
  * @retval -
  * @note   for debug
  */
/*--------------------------------------------------*/
static void port_error(void)
{
    while(1)
    {
        __NOP();
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Variable Initialize
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void variable_initialize(void)
{
    /* Instance. */
}

/*--------------------------------------------------*/
/** 
  * @brief  Clear input data.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void clear_data(void)
{
    uint8_t i;

    for (i=0; i<(UART_IO_RECEIVE_MAX+1); i++)
    {
        input.data[i] = 0;
    }
    input.wp = 0;

    for (i=0; i<UART_IO_RECEIVE_MAX; i++)
    {
        read_data[i] = 0;
    }
}
/**
 *  @}
 */ /* End of group ETHM_IO_PRIVATE */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/

/*--------------------------------------------------*/
/** 
  * @brief  Initialize
  * @param  -
  * @retval -
  * @pre    -
  * @note   -
  */
/*--------------------------------------------------*/
void ethm_io_initialize(void)
{
    /*------------------------------*/
    /*  Clear RAM                   */
    /*------------------------------*/
    send.state      = SEND_IDLE;
    send.wp         = 0;
    send.rp         = 0;
    receive.state   = RECV_RUN;     /* Auto run. */
    receive.wp      = 0;
    receive.rp      = 0;
    receive.length  = 0;

	ETHM_Init(TSB_EMA);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
   TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    /* Tx & Rx use same UART channel. */
    {
        uart_io_initial_setting_t param;

        param.id         = USER_ID_UART;
        switch (bsp_get_usb_uart_tx_ch(BSP_USB_UART_0))
        {
            case 0:
                 param.p_reg  = TSB_UART0;
                 break;
            case 1:
                 param.p_reg  = TSB_UART1;
                 break;
            case 2:
                 param.p_reg  = TSB_UART2;
                 break;
            default:
                 param.p_reg  = MAIN_NULL;
                 break;
        }
        param.boudrate   = bsp_get_usb_uart_tx_baudrate(BSP_USB_UART_0);
        param.irq.tx     = bsp_get_usb_uart_tx_nvic(BSP_USB_UART_0);
        param.irq.rx     = bsp_get_usb_uart_rx_nvic(BSP_USB_UART_0);
        param.irq.err    = bsp_get_usb_uart_err_nvic(BSP_USB_UART_0);
        if (param.p_reg != MAIN_NULL)
        {
            result  = uart_io_initialize(&param);
        }
        else
        {
            result = TXZ_ERROR;
        }
    }

    return (result);
}
/*--------------------------------------------------*/
/** 
  * @brief  Port Initialize.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void ETHM_port_initialize(void)
{
    /*===============================================================================*/
    /*  [Port G]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PGDATA | PGCR | PGFRn | PGOD | PGPUP | PGDN | PGIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PG0  |  EALE     | O   | 0      | 1    | PGFR1 | 0    | 0     | 0    | 0     */
    /*  PG1  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PG2  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PG3  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PG4  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PG5  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PG6  |  PSW 0    | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*  PG7  |  PSW 1    | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pg_instance = TSB_PG;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_G) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PG0 -*/
    if (gpio_func(&port, GPIO_PORT_G, GPIO_PORT_0, (uint32_t)GPIO_PG0_EMARXDV, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PG1 -*/
    if (gpio_func(&port, GPIO_PORT_G, GPIO_PORT_1, (uint32_t)GPIO_PG1_EMARXER, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PG2 -*/
    if (gpio_func(&port, GPIO_PORT_G, GPIO_PORT_2, (uint32_t)GPIO_PG2_EMACRS, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PG3 -*/
    if (gpio_func(&port, GPIO_PORT_G, GPIO_PORT_3, (uint32_t)GPIO_PG3_EMACOL, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PG4 -*/    
    if (gpio_func(&port, GPIO_PORT_G, GPIO_PORT_4, (uint32_t)GPIO_PG4_EMARXD2, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PG5 -*/
    if (gpio_func(&port, GPIO_PORT_G, GPIO_PORT_5, (uint32_t)GPIO_PG5_EMARXD3, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PG7 -*/
    gpio_write_bit(&port,  GPIO_PORT_G, GPIO_PORT_7, GPIO_Mode_DATA, GPIO_PIN_SET);
    if (gpio_func(&port, GPIO_PORT_G, GPIO_PORT_7, (uint32_t)GPIO_PG7_EMAMDC, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    
    /*===============================================================================*/
    /*  [Port H]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PHDATA | PHCR | PHFRn | PHOD | PHPUP | PHDN | PHIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PH0  | UT1TXDA   | O   | 0      | 1    | PHFR5 | 0    | 0     | 0    | 0     */
    /*  PH1  | UT1RXD    | I   | 0      | 0    | PHFR5 | 0    | 0     | 0    | 1     */
    /*  PH2  |  -        | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PH3  | TDI       | -   | 0      | 0    | PHFR1 | 0    | 1     | 0    | 1     */
    /*  PH4  | SWDIO     | -   | 0      | 1    | PHFR1 | 0    | 1     | 0    | 1     */
    /*  PH5  | TCK       | -   | 0      | 0    | PHFR1 | 0    | 0     | 1    | 1     */
    /*  PH6  | TDO       | -   | 0      | 1    | PHFR1 | 0    | 0     | 0    | 0     */
    /*  PH7  | TRST_N    | -   | 0      | 0    | PHFR1 | 0    | 1     | 0    | 1     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_ph_instance = TSB_PH;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_H) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PH0 -*/
    gpio_write_bit(&port,  GPIO_PORT_H, GPIO_PORT_0, GPIO_Mode_DATA, GPIO_PIN_RESET);
    if (gpio_func(&port, GPIO_PORT_H, GPIO_PORT_0, (uint32_t)GPIO_PH0_EMAMDIO, GPIO_PIN_INOUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    
    /*===============================================================================*/
    /*  [Port K]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PKDATA | PKCR | PKFRn | PKOD | PKPUP | PKDN | PKIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PK0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PK1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PK2  | ECS0_N    | O   | 0      | 1    | PKFR1 | 0    | 0     | 0    | 0     */
    /*  PK3  | ECS1_N    | O   | 0      | 1    | PKFR1 | 0    | 0     | 0    | 0     */
    /*  PK4  |PORT_LED[0]| O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*  PK5  |PORT_LED[1]| O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*  PK6  |PORT_LED[2]| O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*  PK7  |PORT_LED[3]| O   | 0      | 1    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pk_instance = TSB_PK;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_K) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PK0 -*/
    gpio_write_bit(&port,  GPIO_PORT_K, GPIO_PORT_0, GPIO_Mode_DATA, GPIO_PIN_RESET);
    if (gpio_func(&port, GPIO_PORT_K, GPIO_PORT_0, (uint32_t)GPIO_PK0_EMATXEN, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
#if defined(TMPM4NN)
    /*- PK4 -*/
    gpio_write_bit(&port,  GPIO_PORT_K, GPIO_PORT_4, GPIO_Mode_DATA, GPIO_PIN_RESET);
    if (gpio_func(&port, GPIO_PORT_K, GPIO_PORT_4, (uint32_t)GPIO_PK4_EMATXCLK, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PK5 -*/
    gpio_write_bit(&port,  GPIO_PORT_K, GPIO_PORT_5, GPIO_Mode_DATA, GPIO_PIN_RESET);
    if (gpio_func(&port, GPIO_PORT_K, GPIO_PORT_5, (uint32_t)GPIO_PK5_EMATXD3, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PK6 -*/
    gpio_write_bit(&port,  GPIO_PORT_K, GPIO_PORT_6, GPIO_Mode_DATA, GPIO_PIN_RESET);
    if (gpio_func(&port, GPIO_PORT_K, GPIO_PORT_6, (uint32_t)GPIO_PK6_EMATXD2, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PK7 -*/
    gpio_write_bit(&port,  GPIO_PORT_K, GPIO_PORT_7, GPIO_Mode_DATA, GPIO_PIN_RESET);
    if (gpio_func(&port, GPIO_PORT_K, GPIO_PORT_7, (uint32_t)GPIO_PK7_EMATXD1, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
#endif
    
    /*===============================================================================*/
    /*  [Port L]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PLDATA | PLCR | PLFRn | PLOD | PLPUP | PLDN | PLIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PL0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PL1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PL2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PL3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PL4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PL5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PL6  | PSW 2     | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*  PL7  | PSW 3     | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pl_instance = TSB_PL;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_L) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
#if defined(TMPM4NN)
    /*- PL0 -*/
    if (gpio_func(&port, GPIO_PORT_L, GPIO_PORT_0, (uint32_t)GPIO_PL0_EMATXD0, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
#endif
    /*- PL1 -*/
    if (gpio_func(&port, GPIO_PORT_L, GPIO_PORT_1, (uint32_t)GPIO_PL1_EMARXCLK, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PL2 -*/
    if (gpio_func(&port, GPIO_PORT_L, GPIO_PORT_2, (uint32_t)GPIO_PL2_EMARXD0, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PL3 -*/
    if (gpio_func(&port, GPIO_PORT_L, GPIO_PORT_3, (uint32_t)GPIO_PL3_EMARXD1, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    
#if (defined(TMPM4NR)||defined(TMPM4NQ))    
    /*===============================================================================*/
    /*  [Port M]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PMDATA | PMCR | PMFRn | PMOD | PMPUP | PMDN | PMIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PM0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PM1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PM2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PM3  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PM4  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PM5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PM6  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PM7  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pm_instance = TSB_PM;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_M) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
    /*- PM0 -*/
    gpio_write_bit(&port,  GPIO_PORT_M, GPIO_PORT_0, GPIO_Mode_DATA, GPIO_PIN_RESET);
    if (gpio_func(&port, GPIO_PORT_M, GPIO_PORT_0, (uint32_t)GPIO_PM0_EMATXD3, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PM1 -*/
    gpio_write_bit(&port,  GPIO_PORT_M, GPIO_PORT_1, GPIO_Mode_DATA, GPIO_PIN_RESET);
    if (gpio_func(&port, GPIO_PORT_M, GPIO_PORT_1, (uint32_t)GPIO_PM1_EMATXD2, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PM2 -*/
    gpio_write_bit(&port,  GPIO_PORT_M, GPIO_PORT_2, GPIO_Mode_DATA, GPIO_PIN_RESET);
    if (gpio_func(&port, GPIO_PORT_M, GPIO_PORT_2, (uint32_t)GPIO_PM2_EMATXD1, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*- PM3 -*/
    gpio_write_bit(&port,  GPIO_PORT_M, GPIO_PORT_0, GPIO_Mode_DATA, GPIO_PIN_RESET);
    if (gpio_func(&port, GPIO_PORT_M, GPIO_PORT_3, (uint32_t)GPIO_PM3_EMATXD0, GPIO_PIN_OUTPUT) != TXZ_SUCCESS)
	{
        port_error();
    }
#endif
    
    /*===============================================================================*/
    /*  [Port T]                                                                     */
    /*-------------------------------------------------------------------------------*/
    /*  Port | Function  | I/O | PTDATA | PTCR | PTFRn | PTOD | PTPUP | PTDN | PTIE  */
    /*-------------------------------------------------------------------------------*/
    /*  PT0  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PT1  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PT2  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*  PT3  | RXIN0     | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*  PT4  | RXIN1     | I   | 0      | 0    | 0     | 0    | 0     | 0    | 1     */
    /*  PT5  | -         | -   | 0      | 0    | 0     | 0    | 0     | 0    | 0     */
    /*===============================================================================*/
    /*--- Construct  ---*/
    /* Register Allocation */
    port.p_pt_instance = TSB_PT;
    /*--- Initialize ---*/
    if (gpio_init(&port, GPIO_PORT_T) != TXZ_SUCCESS)
    {
        port_error();
    }
    /*--- Setting    ---*/
#if (defined(TMPM4NR)||defined(TMPM4NQ))
    /*- PT4 -*/
    if (gpio_func(&port, GPIO_PORT_T, GPIO_PORT_4, (uint32_t)GPIO_PT4_EMATXCLK, GPIO_PIN_INPUT) != TXZ_SUCCESS)
    {
        port_error();
    }
#endif
}

/**
 *  @}
 */ /* End of group MAIN_PRIVATE */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    int ret;
    /*----------------------*/
    /* Set Processing Mode  */
    /*----------------------*/
    proc = PROC_OUTPUT_MSG;
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /* BSP initialize       */
    bsp_initialize();
    
    /* ETHM port initialize */
    ETHM_port_initialize();
    
    /* Valiable initialization */
    variable_initialize();

    /* Application initialization */
    if(application_initialize() != TXZ_SUCCESS)
    {
        proc = PROC_ERROR;
    }
    ethm_io_initialize();
    
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    while(1)
    {
        switch(proc)
        {
            /*--- Output "First Message" ---*/
            case PROC_OUTPUT_MSG:
                  {
                     /* Clear input data.     */
                     clear_data();
                     /* Output first message. */
                     if (printf("command > ") != -1 )
                     {
                           proc = PROC_INPUT;
                     }
                     else
                     {
                           proc = PROC_ERROR;
                     }
                  }
                  break;
            /*--- Wait to Input ---*/
            case PROC_INPUT:
                  {
                    int data;
                    
                    data = getchar();
                    if ((data != -1) && (input.wp < UART_IO_RECEIVE_MAX))
                    {
                          input.data[input.wp] = (char)data;
                          input.wp++;
                          switch ((char)data)
                          {
                          /*--- C.dataR: carriage return ---*/
                          case '\r':
                                /* no process */
                                break;
                          /*--- LF: line feed ---*/
                          case '\n':
                                proc = PROC_COMMAND_PARSE;
                                break;
                          /*--- default ---*/
                          default:
                                /* no process */
                                break;
                          }
                    }
                    else
                    {
                          proc = PROC_OUTPUT_ERR;
                    }
                  }
                  break;
            /*--- Output "Echo Back Data" ---*/
            case PROC_COMMAND_PARSE:
                  {
                    if ((strncmp((const char *)&input.data[0], "write", 5)) == 0)
                    {
						send.buff[ 0] = 0xFF;
						send.buff[ 1] = 0xFF;
						send.buff[ 2] = 0xFF;
						send.buff[ 3] = 0xFF;
						send.buff[ 4] = 0xFF;
						send.buff[ 5] = 0xFF;
						send.buff[ 6] = 0x00;
						send.buff[ 7] = 0x00;
						send.buff[ 8] = 0x00;
						send.buff[ 9] = 0x00;
						send.buff[10] = 0x00;
						send.buff[11] = 0x00;
						send.buff[12] = 0x90;
						send.buff[13] = 0x00;
						send.buff[14] = 'T';
						send.buff[15] = 'O';
						send.buff[16] = 'S';
						send.buff[17] = 'H';
						send.buff[18] = 'I';
						send.buff[19] = 'B';
						send.buff[20] = 'A';
						send.buff[21] = 0x00;
						ETHM_Transmit(TSB_EMA, send.buff, 22);
                          proc = PROC_WRITE_EXEC;
                          break;
                    }
                    else if ((strncmp((const char *)&input.data[0], "read", 4)) == 0)
                    {
					    ETHM_Receive(TSB_EMA, receive.buff, 64);
                          uint32_t i;
                          volatile uint8_t *src_ptr = receive.buff;
                          uint8_t *dst_ptr = read_data;
                          
                          for(i=0;i<ETHM_IO_RECEIVE_MAX;++i)
                          {
                                *dst_ptr++ = *src_ptr++;
                          }
                          
                          (void)printf("read data > ");
                          if(ret != TXZ_ERROR)   {
                                (void)printf("%s", (const char*)&read_data[14]);
                          }
                          (void)printf("\n");
                          proc = PROC_OUTPUT_MSG;
                    }   
                    else
                    {
                          proc = PROC_OUTPUT_ERR;
                    }
                  }
                  break;
                  
            case PROC_WRITE_EXEC:
                  {
                    proc = PROC_OUTPUT_MSG;
                    if(send.state == SEND_IDLE){
                    }
                    else if(send.state == SEND_FAILURE){
                    }
                    else if(send.state == SEND_SUCCESS) {

                    }
                    break;
                }
           /*--- Output "Input Error Message" ---*/
            case PROC_OUTPUT_ERR:
                  {
                    if (printf("\n Command Error !! \n") != -1 )
                    {
                          proc = PROC_OUTPUT_MSG;
                    }
                    else
                    {
                          proc = PROC_ERROR;
                    }
                  }
                  break;
            /*--- Error ---*/
            case PROC_ERROR:
                  /* System Error. Please Debug. */
      #ifdef __DEBUG__
                  assert_failed(MAIN_NULL, 0);
      #endif
                  break;
            /*--- Default ---*/
            default:
                  proc = PROC_ERROR;
                  break;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    switch (uart)
    {
      /*--- usb uart 0 ---*/
      case BSP_USB_UART_0:
          uart_io_rx_irq_handler();
          break;
      /*--- default    ---*/
      default:
          /* no processing */
          break;
    }

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    switch (uart)
    {
      /*--- usb uart 0 ---*/
      case BSP_USB_UART_0:
          uart_io_tx_irq_handler();
          break;
      /*--- default    ---*/
      default:
          /* no processing */
          break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    switch (uart)
    {
      /*--- usb uart 0 ---*/
      case BSP_USB_UART_0:
          uart_io_err_irq_handler();
          break;
      /*--- default    ---*/
      default:
          /* no processing */
          break;
    }
}

#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif

/**
 *  @} 
 */ /* End of group ETHM_LoopBack */

/**
 *  @} 
 */ /* End of group Sample_Appli */

#ifdef __cplusplus
}
#endif /* __cplusplus */
